import sys

class DebugProxy(object):
    '''Dump calls of specified target to console and relay them.'''
    def __init__(self, target):
        self.__target = target
        self.__name = target.__name__
    
    def __getattr__(self, attribute):
        sys.stdout.write('(getattr) %s.%s' % (self.__name, attribute))
        value = getattr(self.__target, attribute)
        sys.stdout.write(' = %r\n' % (value,))
        if callable(value):
            return DebugProxy(value)
        return value
    
    def __setattr__(self, attribute, value):
        if attribute in ('_DebugProxy__target', '_DebugProxy__name'):
            return object.__setattr__(self, attribute, value)
        if isinstance(value, DebugProxy):
            value = value.__target
        sys.stdout.write('(setattr) %s.%s = %r\n' % (self.__name, attribute, value))
        return setattr(self.__target, attribute, value)
    
    def __call__(self, *args, **kw):
        argslist = [repr(arg) for arg in args]
        kwlist = ['%s=%r' % item for item in kw]
        argstext = ', '.join(argslist+kwlist)
        sys.stdout.write('(call) %s(%s)\n' % (self.__name, argstext))
        retval = self.__target(*args, **kw)
        sys.stdout.write('(return) %s(...) = %r\n' % (self.__name, retval))
        return retval
